package require BLT

# Some defaults and constants
set Client(count) 0
#set Client(colors) {red green blue purple orange}
set Client(stipple) {cross2 bdiagonal2 checker2 checker3}
set Client(tics) {{rcv bytes} {xmt bytes} {rcv errs} {xmt errs} {colls}}
set Client(pos) {1          9            3          11         14}

set Client(ip) 192.168.9.1
#set Client(ip) 127.0.0.1
set Client(port) 55556


blt::bitmap define cross2 {
  {8 8} 
  {0xff, 0x88, 0x88, 0x88, 0xff, 0x88, 0x88, 0x88}
  }

blt::bitmap define bdiagonal2 {
  {8 8} 
  {0x08, 0x04, 0x02, 0x01, 0x80, 0x40, 0x20, 0x10}
  }

blt::bitmap define checker3 {
  {8 8} 
  {0x0f, 0x0f, 0x0f, 0x0f, 0xf0, 0xf0, 0xf0, 0xf0}
  }

blt::bitmap define checker2 {
  {8 8} 
  {0x33, 0x33, 0xcc, 0xcc, 0x33, 0x33, 0xcc, 0xcc}
  }



################################################################
# proc getNetInfo {channel}--
#    Retrieves a set of network information from the socket
#    Parses the info, and creates a set of 'diff' index arrays
#    that are the difference between this value and the previous
#    value for a field.
#
# Arguments
#   channel	The channel to read data from
# 
# Results
#   Modifies the Client array.
#   Invokes processData to update the bar

proc getNetInfo {channel} {
    global Client
    global DataVectors

    gets $channel line

    # The first element is the name, but the second should
    #   be a number.  If it isn't (this is a line of column headers.)
    #   We'll skip out and wait for the next line of data.

    if {[catch {expr [lindex $line 1] * 2}]} {
	return
    }

    # Long integers may run into the ":" in the line label
    #  This gives us a space to parse on.

    regsub ":" $line " " line

    # The first entry is the device name.

    set name [lindex $line 0]
    
    # If there's no DataVectors($name.y), the vectors have not
    # been created for this device.

    if {![info exists DataVectors($name.y)]} {
	makeNewBarSet $name
    }
    
    # The vectors exist in global scope.  Declare them as such.

    foreach v [array names DataVectors $name*] {
        global $DataVectors($v)
    }

    # The DataVectors($name.y) vector holds the heights of the bars
    #   for a given data set.

    set vectorPos 0
    foreach pos $Client(pos) label $Client(tics) {
	set val [lindex $line $pos]
	if {[info exists Client($name.$label)]} {
	    set $DataVectors($name.y)($vectorPos) \
	        [expr $val - $Client($name.$label)]
	    incr vectorPos
	}
	set Client($name.$label) $val
    }
    
    foreach dir {rcv xmt} val [list [lindex $line 1] [lindex $line 9]] {
      if {[info exists Client($name.$dir)]} {
        set v2 [expr $val - $Client($name.$dir)]
        unset $DataVectors($name.y.$dir)(0)
        set $DataVectors($name.y.$dir)(0)
        set $DataVectors($name.y.$dir)(++end) $v2
      }
    set Client($name.$dir) $val
    }
    
    set Client(update) "Last Update: [clock format [clock seconds]\
      -format %H:%M:%S]"
}

################################################################
# proc makeNewBarSet {name}--
#  Creates a new set of bars, and reconfigures the barchart to hold them.
#
# Arguments
#  name		The name of the data associated with this set
# 
# Results
#   Creates new global vectors.


proc uniq {{val 0}} {
  incr val
  proc uniq "{val $val}" [info body uniq]
  return $val
}

proc makeNewBarSet {name} {
    global Client
    global DataVectors

puts "Building newBarSet for $name [array get DataVectors]"
    # Generate a set of unique vector names, and
    # save those names in the DataVectors associative array

    array set DataVectors [list \
        $name.y vector[uniq]  \
        $name.y.rcv vector[uniq] $name.y.xmt vector[uniq] ]
    
    # Declare each of the new vectors to exist in global scope

    foreach v [array names DataVectors $name*] {
        global $DataVectors($v)
    }
    
    # Create the new vectors

    ::blt::vector $DataVectors($name.y)(5)
    ::blt::vector $DataVectors($name.y.rcv)(200)
    ::blt::vector $DataVectors($name.y.xmt)(200)

    
    # And initialize the history vectors

    for {set i 0} {$i < 200} {incr i} {
        set $DataVectors($name.y.rcv)($i) 0
        set $DataVectors($name.y.xmt)($i) 0
    }
    
    # Create the new current data elements

    $Client(barChart) element create $name -label "$name" \
        -stipple [lindex $Client(stipple) $Client(count)] \
	-fg black -bg white \
	-xdata dataXvector \
	-ydata $DataVectors($name.y)

    # Create the new history elements.
    #  Use inverted colors for the XMIT and RECEIVE elements

    set inverse(black) white
    set inverse(white) black
    set color white

    foreach dir {rcv xmt} {
      $Client(historyChart) element create ${dir}_$name -label "" \
        -stipple [lindex $Client(stipple) $Client(count)] \
	-fg $color -bg $inverse($color) -borderwidth 0 \
	-xdata historyXvector -ydata $DataVectors($name.y.$dir)
      set color $inverse($color)
    }

    incr Client(count)
}

################################################################
# proc getTicLabel {chart tic}--
#    Returns a textual label for the barchart
# Arguments
#   chart	The chart associated with this request
#   tic		The position of the tic being requested.

proc getTicLabel {chart tic} {
    global Client

    return [lindex $Client(tics) $tic]
}

# Make a quit button
button .b -text "Quit" -command "exit"
grid .b -row 0 -column 0

# And the update time label
label .l -textvar Client(update)
grid .l -row 0 -column 1

# Build a barchart for current values

set Client(barChart) [::blt::barchart .bcht -width 600 -title\
  "Network Activity" -barmode aligned]
$Client(barChart) axis configure x -command getTicLabel
$Client(barChart) axis configure y -logscale 1

grid $Client(barChart) -row 2 -column 0 -columnspan 3

# Build canvas, and then place the history barchart inside it

canvas .historyCvs -height 120 -width 600 -xscrollcommand {.histSB set}
grid .historyCvs -row 3 -column 0 -columnspan 4

scrollbar .histSB -orient horizontal -command {.historyCvs xview}
grid .histSB -row 4 -column 0 -columnspan 4 -sticky ew

set Client(historyChart) [::blt::barchart .historyCvs.bhist -width 8000 \
  -barmode aligned -title "" -height 150 -barmode aligned]

.historyCvs create window 1 1 -window $Client(historyChart) -anchor nw

$Client(historyChart) legend configure -hide yes
$Client(historyChart) axis configure x -showticks 0
$Client(historyChart) axis configure y -logscale 1
$Client(historyChart) grid configure -minor 0


# Build and initialize the vectors for the X data in the 
#  current data and historical data barcharts.

::blt::vector dataXvector(5)
for {set i 0} {$i < 5} {incr i} {
   set dataXvector($i) $i
}

::blt::vector historyXvector(200)
for {set i 0} {$i < 200} {incr i} {
   set historyXvector($i) $i
}

update idle; update;

puts "BBOX: [.historyCvs bbox all]"
.historyCvs configure -scrollregion [.historyCvs bbox all]

# Open a client socket on the local system 
# (for testing purposes.)
set input [socket $Client(ip) $Client(port)]

# When data is available to be read, call getNetInfo
fileevent $input readable "getNetInfo $input"

# And wait for the fireworks to start.
